/*
 * Copyright 2010 Red Hat, Inc.
 * Red Hat licenses this file to you under the Apache License, version
 * 2.0 (the "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.hornetq.tests.integration.jms.cluster;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.Topic;
import javax.naming.NamingException;

import junit.framework.Assert;

import org.hornetq.api.core.SimpleString;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.jms.HornetQJMSClient;
import org.hornetq.api.jms.JMSFactoryType;
import org.hornetq.core.config.ClusterConnectionConfiguration;
import org.hornetq.core.config.Configuration;
import org.hornetq.core.logging.Logger;
import org.hornetq.core.remoting.impl.invm.InVMAcceptorFactory;
import org.hornetq.core.remoting.impl.invm.InVMConnectorFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.core.server.HornetQServers;
import org.hornetq.core.settings.impl.AddressFullMessagePolicy;
import org.hornetq.core.settings.impl.AddressSettings;
import org.hornetq.jms.client.HornetQConnectionFactory;
import org.hornetq.jms.server.config.impl.JMSConfigurationImpl;
import org.hornetq.jms.server.impl.JMSServerManagerImpl;
import org.hornetq.tests.integration.cluster.distribution.ClusterTestBase;
import org.hornetq.tests.unit.util.InVMContext;
import org.hornetq.tests.util.ServiceTestBase;

/**
 * A ClusterTestWithPaging
 *
 *
 *
 */
public class QueueClusterWithPagingTest extends ServiceTestBase
{


   private static final Logger log = Logger.getLogger(ClusterTestBase.class);

   protected HornetQServer server1;

   protected JMSServerManagerImpl jmsServer1;

   protected HornetQServer server2;

   protected JMSServerManagerImpl jmsServer2;

   protected ConnectionFactory cf1;

   protected ConnectionFactory cf2;

   protected InVMContext context1;

   protected InVMContext context2;
   
   private static final int MAX_HOPS = 1;




   
   
   /**
    * Consume from server2
    * @throws Exception
    */
   public void testClusteredQueue1() throws Exception
   {

      //This value is 6000 => not paging 
      //              7000 => paging 
      final int numMessages = 7000;

      System.out.println("Server1 = " + server1.getNodeID());
      System.out.println("Server2 = " + server2.getNodeID());
      jmsServer1.createQueue(true, "target", null, true, "/queue/target");
      jmsServer2.createQueue(true, "target", null, true, "/queue/target");

      
      
      
      
      Connection conn1 = cf1.createConnection();
      Connection conn2 = cf2.createConnection();

      conn1.start();
      
      conn2.start();

      try
      {
         Session session1 = conn1.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue targetQueue1 = session1.createQueue("target");


         // sleep a little bit to have the temp queue propagated to server #2
         Thread.sleep(3000);
         MessageProducer prod1 = session1.createProducer(targetQueue1);




         for (int i = 0; i < numMessages; i++)
         {
            BytesMessage msg = session1.createBytesMessage();
            msg.setIntProperty("SendIndex", i);
            msg.writeBytes(getContent());
            prod1.send(msg);
            System.out.println(">>>> " + i);
         }



      }
      finally
      {
         conn1.close();
      }
      
      System.out.println("Paging:" + server1.getPagingManager().getPageStore(new SimpleString("jms.queue.target")).isPaging());
      
      try
      {
         Session session2 = conn2.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue targetQueue2 = session2.createQueue("target");
         MessageConsumer cons2 = session2.createConsumer(targetQueue2);
   
         for (int i = 0; i < numMessages; i++)
         {
            Message r1 = cons2.receive(15000);
            System.out.println("<<< " + i + "/SendIndex:" + r1.getIntProperty("SendIndex"));
            Assert.assertNotNull("did not receive message " + i, r1);
            r1.acknowledge();
         }
      }
      finally
      {
         conn2.close();
      }
      
      
   }





   /**
    * Consume from server1
    * @throws Exception
    */
   public void testClusteredQueue2() throws Exception
   {

      //This value is 6000 => not paging 
      //              7000 => paging 
      final int numMessages = 7000;

      System.out.println("Server1 = " + server1.getNodeID());
      System.out.println("Server2 = " + server2.getNodeID());
      jmsServer1.createQueue(true, "target", null, true, "/queue/target");
      jmsServer2.createQueue(true, "target", null, true, "/queue/target");

      
      
      
      
      Connection conn1 = cf1.createConnection();

      conn1.start();
      

      try
      {
         Session session1 = conn1.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue targetQueue1 = session1.createQueue("target");


         // sleep a little bit to have the temp queue propagated to server #2
         Thread.sleep(3000);
         MessageProducer prod1 = session1.createProducer(targetQueue1);




         for (int i = 0; i < numMessages; i++)
         {
            BytesMessage msg = session1.createBytesMessage();
            msg.setIntProperty("SendIndex", i);
            msg.writeBytes(getContent());
            prod1.send(msg);
            System.out.println(">>>> " + i);
         }

         session1.close();

         System.out.println("Paging:" + server1.getPagingManager().getPageStore(new SimpleString("jms.queue.target")).isPaging());

         Session session2 = conn1.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue targetQueue2 = session2.createQueue("target");
         MessageConsumer cons2 = session2.createConsumer(targetQueue2);
   
         
         for (int i = 0; i < numMessages; i++)
         {
            Message r1 = cons2.receive(15000);
            System.out.println("<<< " + i + "/SendIndex:" + r1.getIntProperty("SendIndex"));
            Assert.assertNotNull("did not receive message " + i, r1);
            r1.acknowledge();
         }
      }
      finally
      {
         conn1.close();
      }
      
      
   }






   // Protected -----------------------------------------------------

 
   @Override
   protected void setUp() throws Exception
   {
      super.setUp();

      setupServer2();
      setupServer1();

      jmsServer1.start();
      jmsServer1.activated();

      jmsServer2.start();
      jmsServer2.activated();

      cf1 = (ConnectionFactory) HornetQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                                generateInVMParams(0)));
      cf2 = (ConnectionFactory)HornetQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                                generateInVMParams(1)));
   }

   /**
    * @param toOtherServerPair
    * @throws Exception
    */
   private void setupServer2() throws Exception
   {
      List<String> toOtherServerPair = new ArrayList<String>();
      toOtherServerPair.add("toServer1");

      Configuration conf2 = createDefaultConfig(1, generateInVMParams(1), InVMAcceptorFactory.class.getCanonicalName());
      conf2.setSecurityEnabled(false);
      conf2.setJMXManagementEnabled(true);
      conf2.setPersistenceEnabled(false);

      conf2.getConnectorConfigurations().put("toServer1",
                                             new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                        generateInVMParams(0)));
      conf2.getConnectorConfigurations().put("server2",
                                             new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                        generateInVMParams(1)));

      conf2.setClustered(true);
      
      conf2.getClusterConfigurations().add(new ClusterConnectionConfiguration("to-server1",
                                                                              "jms",
                                                                                 "server2",
                                                                              1000,
                                                                              true,
                                                                              false,
                                                                              MAX_HOPS,
                                                                              1024,
                                                                              toOtherServerPair, false));


      JMSConfigurationImpl jmsconfig = new JMSConfigurationImpl();
      //jmsconfig.getTopicConfigurations().add(new TopicConfigurationImpl("t1", "topic/t1"));
      

      server2 = HornetQServers.newHornetQServer(conf2, false);
      jmsServer2 = new JMSServerManagerImpl(server2, jmsconfig);
      context2 = new InVMContext();
      jmsServer2.setContext(context2);
      
      AddressSettings addressSettings = jmsServer2.getAddressSettings("#");
      addressSettings.setRedistributionDelay(0);
      addressSettings.setMaxSizeBytes(10485760); // 1 MiB
      addressSettings.setPageSizeBytes(1048576); // 100 kiB
      addressSettings.setAddressFullMessagePolicy(AddressFullMessagePolicy.PAGE);
      
   }

   /**
    * @param toOtherServerPair
    * @throws Exception
    */
   private void setupServer1() throws Exception
   {
      List<String> toOtherServerPair = new ArrayList<String>();
      toOtherServerPair.add("toServer2");

      Configuration conf1 = createDefaultConfig(0, generateInVMParams(0), InVMAcceptorFactory.class.getCanonicalName());
      
      conf1.setSecurityEnabled(false);
      conf1.setJMXManagementEnabled(true);
      conf1.setPersistenceEnabled(false);

      conf1.getConnectorConfigurations().put("toServer2",
                                             new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                        generateInVMParams(1)));
      conf1.getConnectorConfigurations().put("server1",
                                             new TransportConfiguration(InVMConnectorFactory.class.getName(),
                                                                        generateInVMParams(0)));

      conf1.setClustered(true);

      conf1.getClusterConfigurations().add(new ClusterConnectionConfiguration("to-server2",
                                                                              "jms",
                                                                              "server1",
                                                                              1000,
                                                                              true,
                                                                              false,
                                                                              MAX_HOPS,
                                                                              1024,
                                                                              toOtherServerPair, false));

      
      JMSConfigurationImpl jmsconfig = new JMSConfigurationImpl();
      //jmsconfig.getTopicConfigurations().add(new TopicConfigurationImpl("t1", "topic/t1"));
      
      server1 = HornetQServers.newHornetQServer(conf1, false);
      jmsServer1 = new JMSServerManagerImpl(server1, jmsconfig);
      context1 = new InVMContext();
      jmsServer1.setContext(context1);
      
      AddressSettings addressSettings = jmsServer1.getAddressSettings("#");
      addressSettings.setRedeliveryDelay(0);
      addressSettings.setRedistributionDelay(0);
      addressSettings.setMaxSizeBytes(10485760); // 1 MiB
      addressSettings.setPageSizeBytes(1048576); // 100 kiB
      addressSettings.setAddressFullMessagePolicy(AddressFullMessagePolicy.PAGE);
      
   }

   @Override
   protected void tearDown() throws Exception
   {

      try
      {
         jmsServer2.stop();

         server2.stop();

         context2.close();
      }
      catch (Throwable e)
      {
         log.warn("Can't stop server2", e);
      }
      
      ((HornetQConnectionFactory)cf1).close();
      
      ((HornetQConnectionFactory)cf2).close();

      server2 = null;

      jmsServer2 = null;

      context2 = null;

      cf1 = null;

      try
      {
         jmsServer1.stop();

         server1.stop();

         context1.close();
      }
      catch (Throwable e)
      {
         log.warn("Can't stop server2", e);
      }

      server1 = null;

      jmsServer1 = null;

      context1 = null;

      super.tearDown();
   }

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

   protected Map<String, Object> generateInVMParams(final int node)
   {
      Map<String, Object> params = new HashMap<String, Object>();

      params.put(org.hornetq.core.remoting.impl.invm.TransportConstants.SERVER_ID_PROP_NAME, node);

      return params;
   }

   /**
    * 
    */
   private byte[] getContent()
   {
      byte[] content = new byte[2048]; // 2 kiB
      for (int i=0; i < content.length; ++i) {
          content[i] = (byte) i;
      }
      
      return content;
   }
}
